import type { JSONResponse } from '~~/types'

// Composable to make user management tasks easier
export default function useUsers() {
  const apiPath = useRuntimeConfig().public.apiPath

  return {
    getUsers,
    fetchUsers,
    getUsersAnalytics,
    getUser,
    updateUser,
    deleteUser,
    deleteUsers,
    deleteUser2FA,
    updateUsersStatus,
  }

  /**
   * @desc Get users
   * @returns {Promise<JSONResponse>}
   */
  async function getUsers(params = {}): Promise<JSONResponse> {
    const response = await $fetch(apiPath + '/api/admin/users', {
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
      params,
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Fetch users
   * @returns {Promise<JSONResponse>}
   */
  async function fetchUsers(): Promise<JSONResponse> {
    const response = await $fetch(apiPath + '/api/admin/users/fetch/all', {
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Get users analytics
   * @returns {Promise<JSONResponse>}
   */
  async function getUsersAnalytics(): Promise<JSONResponse> {
    const response = await $fetch(apiPath + '/api/admin/users/analytics/all', {
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Get user
   * @returns {Promise<JSONResponse>}
   */
  async function getUser(uuid: string): Promise<JSONResponse> {
    const response = await $fetch(apiPath + `/api/admin/users/${uuid}`, {
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Update a user
   * @param uuid User's uuid
   * @param values User record's editable values
   * @returns {Promise<JSONResponse>}
   */
  async function updateUser(
    uuid: string,
    first_name: string,
    last_name: string,
    email: string,
    role_id: string,
  ): Promise<JSONResponse> {
    const response = await $fetch(apiPath + `/api/admin/users/${uuid}`, {
      method: 'PUT',
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
      body: {
        first_name,
        last_name,
        email,
        role_id,
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Delete a user
   * @uuid User uuid
   * @returns {Promise<JSONResponse>}
   */
  async function deleteUser(uuid: string): Promise<JSONResponse> {
    const response = await $fetch(apiPath + `/api/admin/users/${uuid}`, {
      method: 'DELETE',
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Delete users
   * @users User ids
   * @returns {Promise<JSONResponse>}
   */
  async function deleteUsers(users: number[]): Promise<JSONResponse> {
    const response = await $fetch(apiPath + '/api/admin/users/delete/bulk', {
      method: 'DELETE',
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
      body: {
        users: users,
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Update users status
   * @param users User ids
   * @param status Status to update
   * @returns {Promise<JSONResponse>}
   **/
  async function updateUsersStatus(
    users: number[],
    status: string,
  ): Promise<JSONResponse> {
    const response = await $fetch(
      apiPath + '/api/admin/users/update-status/bulk',
      {
        method: 'PUT',
        credentials: 'include',
        headers: {
          'client-platform': 'browser',
        },
        body: {
          users: users,
          status: status,
        },
      },
    )

    if (!response.status) {
      throw response
    }

    return response
  }

  /**
   * @desc Delete user 2FA
   * @param uuid User uuid
   * @returns {Promise<JSONResponse>}
   **/
  async function deleteUser2FA(uuid: string): Promise<JSONResponse> {
    const response = await $fetch(apiPath + `/api/admin/users/${uuid}/2fa`, {
      method: 'DELETE',
      credentials: 'include',
      headers: {
        'client-platform': 'browser',
      },
    })

    if (!response.status) {
      throw response
    }

    return response
  }
}
